#include <sdcc-lib.h>
#include <pic16f688.h>	// /usr/share/sdcc/non-free/include/pic14/pic16f688.h

#include "inline.h"
#include "libwait.h"
#include "libusart.h"
#include "libirsig.h"

__code unsigned char __at 0x2000 id_location[] = {'A', 'B', 'C', 'D'};

#ifndef _BOR_ON
#define _BOR_ON _BOD_ON
#endif

unsigned int __at 0x2007 __config =
	_FCMEN_OFF &
	_IESO_OFF &
	_BOR_ON &	// for 2.1V
	_CPD_OFF &
	_CP_OFF &
	_MCLRE_OFF &
	_PWRTE_OFF &
	_WDT_OFF &
	_INTRC_OSC_NOCLKOUT;

__code unsigned char __at 0x2100 eedata[] = {'d', 'u', 'm', 'm', 'y', 0};

#define	LED1	RA0

void Intr(void) __interrupt 0 {
	T0IF = 0;						// Clear timer interrupt
}

/*			  _______
	Vcc		-|	 U	 |-	GND
	NC		-|5		0|-	LED1
	NC		-|4	 A	1|-	NC
	NC		-|3		2|-	NC
	RX		-|5		0|-	IRSNS
	TX		-|4	 C	1|-	IRLED
	NC		-|3_____2|-	NC
*/

void main(void) {
	unsigned char i;

				//  76543210
	PORTA		= 0b00000000;		// RA3: input only
	CMCON0		= 0b00000111;
	ANSEL		= 0b00000000;
	TRISA		= 0b00001000;		// 1: 入力, 0: 出力

	PORTC		= 0b00000000;
	TRISC		= 0b00110001;		// 1: 入力, 0: 出力

	OPTION_REG	= 0b11111111;		// 初期値:0b11111111
									// 7:PORTAプルアップ(0:使用する)
									// 6:RA2割り込みポート変化選択(0:立下り)
									// 5:TMR0クロックソース選択ビット(0:内部)
									// 4:TMR0加算タイミング(0:立上り)
									// 3:プリスケーラ割り当てビット(0:TMR0)
									// 2~0プリスケーラレート選択ビット(011:1/16)
									// 4MHz => 1MHz = 1,000,000 / 16 / 256 = 244Hz

	INTCON		= 0b00000000;		// 初期値:0b0000000x
									// 7:GIE  Global Interrupt Enable bit
									// 6:PEIE Peripheral Interrupt Enable bit
									// 5:T0IE TMR0 Overflow Interrupt Enable bit
									// 4:INTE RA2/INT External Interrupt Enable bit
									// 3:RAIE RA Port Change Interrupt Enable bit
									// 2:T0IF TMR0 Overflow Interrupt Flag bit
									// 1:INTF RA2/INT External Interrupt Flag bit
									// 0:RAIF RA Port Change Interrupt Flag bit

	PIR1		= 0b00000000;		// 初期値:0b00000000(フラグ)
	PIE1		= 0b00000000;		// 初期値:0b00000000(イネーブル)
									// 7:EEIE EE Write Complete Interrupt Enable Bit
									// 6:ADIE A/D Converter Interrupt Enable bit
									// 5:RCIE USART Receive Interrupt Enable bit
									// 4:C2IE Comparator 2 Interrupt Enable bit
									// 3:C1IE Comparator 1 Interrupt Enable bit
									// 2:OSFIE Oscillator Fail Interrupt Enable bit
									// 1:TXIE EUSART Transmit Interrupt Enable bit
									// 0:TMR1IE TMR1 Overflow Interrupt Enable bit

	T1CON		= 0b00000000;		// 初期値:0b00000000
									// 7:T1GINV Timer1 Gate Invert bit
									// 6:TMR1GE Timer1 Gate Enable bit
									// 5~4:T1CKPS Timer1 Input Clock Prescale Select bits(00:1/1, 11:1/8)
									// 3:T1OSCEN Timer1 Oscillator Enable Control bit(1:Enable)
									// 2:T1SYNC Timer1 External Clock Input Synchronization Control bit(1:Sync)
									// 1:TMR1CS Timer1 Clock Source Select bit(0:Internal)
									// 0:TMR1ON Timer1 On bit(1:Enable)

//	forCalibOsc();
	initUsart();

	//--------------------------------------------------------------------------
	//
	//	Main Loop
	//
	while(1) {
		LED1 = 1;
		transmitUsart(0x55);
		waitL(5);

/*		clearIrSig();
		while(IRSNS);
		receiveIrSig();
		for(i = 0; i < IRSIGMAX; i++) {
			transmitUsart(irsig[i]);
		}
*/
		irsig[0] = 0x2A;			// DENON CEOL RCD-N9 VOLUME DOWN
		irsig[1] = 0x4C;
		irsig[2] = 0x0A;
		irsig[3] = 0x8A;
		irsig[4] = 0x02;
		irsig[5] = 0x82;
		irsig[6] = 0xFF;
		transmitIrSig();

		LED1 = 0;
		transmitUsart(0xAA);
		waitL(10);
	}
}

