#include "libirsig.h"
#include "libwait.h"

unsigned char irsig[IRSIGMAX];
static unsigned char sig_p;
static unsigned char sig_b;

/*------------------------------------------------------------------------------
 *
 *	赤外線信号バッファクリア
 *
 */
void clearIrSig() {

	for(sig_p = 0; sig_p < IRSIGMAX; sig_p++)	// clear signal buf
		irsig[sig_p] = 0;

	sig_b = -1; sig_p = 0;						// 最初のビットを捨てる
}

//	Example: AEHA IR signal format
// 
//		A		B		C				A		B						C
//	----+		+-------+	H		----+		+-----------------------+	H
//		|		|		|				|		|						|
//		+-------+		+---L			+-------+						+---L
//	0:	  0.4ms	  0.4ms				1:	  0.4ms			  1.2ms

/*------------------------------------------------------------------------------
 *
 *	赤外線信号受信処理
 *
 *	clearIrSig();
 *	while(IRSNS);								// wait Falling edge of IRSNS (A)
 *	receiveIrSig();
 */
void receiveIrSig() {

	unsigned char sig_span;

	while(1) {
		while(!IRSNS);							// wait Raising edge of IRSNS (B)

		sig_span = 0;
		while(IRSNS) {							// wait Falling edge of IRSNS (C)
			waitS(2);							// 2 x 3 + 13 = 19us 消費
			if(++sig_span > 200) break;
		}
#if 0											// 1: for timing check
		irsig[sig_p] = sig_span;
		sig_p++;
#else
		irsig[sig_p] <<= 1;
		if(sig_span > 30)
			irsig[sig_p] |= 1;					// check Active (long span) signal

		if(++sig_b == 8) {
			sig_b = 0;
			sig_p++;
		}
#endif
		if(sig_span > 200 || sig_p == IRSIGMAX) break;
	}
	irsig[sig_p] = 0xFF;						// FIX: over write last byte
}

/*------------------------------------------------------------------------------
 *
 *	赤外線信号送信処理
 *
 */
void transmitIrSig() {

	__asm
		movlw	_irsig			;							LD		A, _irsig
		movwf	FSR			;							LD		IX, A
IrSig:						;			IrSig:											; leader
		movlw	133			;							LD		A, 133					; 3500us / 26.3us = 133
		movwf	_sig_p			;							LD		(_sig_p), A
		call	IrActive		;							CALL	IrActive
		movlw	65			;							LD		A, 65					; 1700us / 26.3us =  65
		movwf	_sig_p			;							LD		(_sig_p), A
		call	IrInactive		;							CALL	IrInactive
IrSig1:						;			IrSig1:
		movlw	0xFF			;							LD		A, 0xFF
		subwf	INDF, W			;							SUB		(IX), A>A
		btfsc	STATUS, 2		
		goto	IrSig9			;							JP		Z, IrSig9				; Z: bit 2
						;	
		movlw	8			;							LD		A, 8
		movwf	_sig_b			;							LD		(_sig_b), A
IrSig2:						;			IrSig2:
		rlf	INDF, F			;							RL		(IX)
		btfsc	STATUS, 0		
		goto	IrSig3			;							JP		C, IrSig3				; C: bit 0
						;															; ---- '0' signal ----
						;	;						LD		A, 18					;  480us / 26.3us =  18
		movlw	15			;							LD		A, 15					;  400us / 26.3us =  15
		movwf	_sig_p			;							LD		(_sig_p), A
		call	IrActive		;							CALL	IrActive
						;	;						LD		A, 14					;  380us / 26.3us =  14
		movlw	21			;							LD		A, 21					;  550us / 26.3us =  21
		movwf	_sig_p			;							LD		(_sig_p), A
		call	IrInactive		;							CALL	IrInactive
		goto	IrSig4			;							JP		IrSig4
IrSig3:						;			IrSig3:											; ---- '1' signal ----
						;	;						LD		A, 18					;  480us / 26.3us =  18
		movlw	15			;							LD		A, 15					;  400us / 26.3us =  15
		movwf	_sig_p			;							LD		(_sig_p), A
		call	IrActive		;							CALL	IrActive
						;	;						LD		A, 47					; 1230us / 26.3us =  47
		movlw	55			;							LD		A, 55					; 1450us / 26.3us =  55
		movwf	_sig_p			;							LD		(_sig_p), A
		call	IrInactive		;							CALL	IrInactive
IrSig4:						;			IrSig4:
		decfsz	_sig_b, F		
		goto	IrSig2			;							DJNZ	(_sig_b), IrSig2
		incf	FSR, F			;							INC		IX
		goto	IrSig1			;							JP		IrSig1
IrSig9:						;			IrSig9:											; trailer
						;	;						LD		A, 18					;  480us / 26.3us =  18
		movlw	15			;							LD		A, 15					;  400us / 26.3us =  15
		movwf	_sig_p			;							LD		(_sig_p), A
		call	IrActive		;							CALL	IrActive
		return				;							RET
						;	
IrActive:					;			IrActive:										; 38kHz = 26.32us
		bsf	_IR_PORT, _IR_LEDBIT	;							SET		_IR_LEDBIT, (_IR_PORT)	;  0
		nop				;							NOP								;  1
		nop				;							NOP								;  2
		nop				;							NOP								;  3
		nop				;							NOP								;  4
		nop				;							NOP								;  5
		nop				;							NOP								;  6
		nop				;							NOP								;  7
		bcf	_IR_PORT, _IR_LEDBIT	;							RES		_IR_LEDBIT, (_IR_PORT)	;  8
		nop				;							NOP								;  9
		nop				;							NOP								; 10
		nop				;							NOP								; 11
		nop				;							NOP								; 12
		nop				;							NOP								; 13
		nop				;							NOP								; 14
		nop				;							NOP								; 15
		nop				;							NOP								; 16
		nop				;							NOP								; 17
		nop				;							NOP								; 18
		nop				;							NOP								; 19
		nop				;							NOP								; 20
		nop				;							NOP								; 21
		nop				;							NOP								; 22
		decfsz	_sig_p, F		
		goto	IrActive		;							DJNZ	(_sig_p), IrActive		; 23, 24, 25
		return				;							RET
						;	
IrInactive:					;			IrInactive:										; 38kHz = 26.32us
		nop				;							NOP								;  0
		nop				;							NOP								;  1
		nop				;							NOP								;  2
		nop				;							NOP								;  3
		nop				;							NOP								;  4
		nop				;							NOP								;  5
		nop				;							NOP								;  6
		nop				;							NOP								;  7
		nop				;							NOP								;  8
		nop				;							NOP								;  9
		nop				;							NOP								; 10
		nop				;							NOP								; 11
		nop				;							NOP								; 12
		nop				;							NOP								; 13
		nop				;							NOP								; 14
		nop				;							NOP								; 15
		nop				;							NOP								; 16
		nop				;							NOP								; 17
		nop				;							NOP								; 18
		nop				;							NOP								; 19
		nop				;							NOP								; 20
		nop				;							NOP								; 21
		nop				;							NOP								; 22
		decfsz	_sig_p, F		
		goto	IrInactive		;							DJNZ	(_sig_p), IrInactive	; 23, 24, 25
		return				;							RET
	__endasm;
}

